@@ -4,6 +4,7 @@ use warnings;
 our (%gui, %vmc);
 
 sub setup_edit_dialog_ioports {
+    my $port = &getsel_combo($gui{comboboxEditIOSelectedSerial}, 1);
     &addrow_log("Fetching I/O port settings for $vmc{Name}...");
     &busy_pointer($gui{dialogEdit}, 1);
     $gui{checkbuttonEditParallelEnable}->set_active(&bl(IParallelPort_getEnabled($vmc{IParallelPort})));
@@ -25,50 +26,53 @@ sub setup_edit_dialog_ioports {
         $gui{entryEditParallelIO}->set_sensitive(1);
     }
 
-    foreach my $slot (1..2) {
-        my $ISerialPort = IMachine_getSerialPort($vmc{IMachine}, ($slot - 1));
-        my $irq = ISerialPort_getIRQ($ISerialPort);
-        my $ioport = sprintf('%X', ISerialPort_getIOBase($ISerialPort));
-        my $mode = ISerialPort_getHostMode($ISerialPort);
-        $gui{'checkbuttonEditSerialEnable' . $slot}->set_active(&bl(ISerialPort_getEnabled($ISerialPort)));
-        $gui{'tableEditSerial' . $slot}->set_sensitive($gui{'checkbuttonEditSerialEnable' . $slot}->get_active()); # Ghost/Unghost other widgets based on serial enabled
-        $gui{'checkbuttonEditSerialPipe' . $slot}->set_active(&bl(ISerialPort_getServer($ISerialPort)));
-        $gui{'entryEditSerialPath' . $slot}->set_text(ISerialPort_getPath($ISerialPort));
-        $gui{'entryEditSerialIRQ' . $slot}->set_text($irq);
-        $gui{'entryEditSerialIO' . $slot}->set_text($ioport);
-        $gui{'entryEditSerialIRQ' . $slot}->set_sensitive(0);
-        $gui{'entryEditSerialIO' . $slot}->set_sensitive(0);
+    my $ISerialPort = IMachine_getSerialPort($vmc{IMachine}, $port);
+    my $irq = ISerialPort_getIRQ($ISerialPort);
+    my $ioport = sprintf('%X', ISerialPort_getIOBase($ISerialPort));
+    my $mode = ISerialPort_getHostMode($ISerialPort);
+    $gui{checkbuttonEditSerialEnable}->set_active(&bl(ISerialPort_getEnabled($ISerialPort)));
+    $gui{tableEditSerial}->set_sensitive($gui{checkbuttonEditSerialEnable}->get_active()); # Ghost/Unghost other widgets based on serial enabled
+    $gui{checkbuttonEditSerialPipe}->set_active(!&bl(ISerialPort_getServer($ISerialPort)));
+    $gui{entryEditSerialPath}->set_text(ISerialPort_getPath($ISerialPort));
+    $gui{entryEditSerialIRQ}->set_text($irq);
+    $gui{entryEditSerialIO}->set_text($ioport);
+    $gui{entryEditSerialIRQ}->set_sensitive(0);
+    $gui{entryEditSerialIO}->set_sensitive(0);
 
-        if ($irq == 4 and uc($ioport) eq '3F8') { $gui{'comboboxEditSerialPortNum' . $slot}->set_active(0); }
-        elsif ($irq == 3 and uc($ioport) eq '2F8') { $gui{'comboboxEditSerialPortNum' . $slot}->set_active(1); }
-        elsif ($irq == 4 and uc($ioport) eq '3E8') { $gui{'comboboxEditSerialPortNum' . $slot}->set_active(2); }
-        elsif ($irq == 3 and uc($ioport) eq '2E8') { $gui{'comboboxEditSerialPortNum' . $slot}->set_active(3); }
-        else {
-            $gui{'comboboxEditSerialPortNum' . $slot}->set_active(4);
-            $gui{'entryEditSerialIRQ' . $slot}->set_sensitive(1);
-            $gui{'entryEditSerialIO' . $slot}->set_sensitive(1);
-        }
-
-        if ($mode eq 'HostPipe') { $gui{'comboboxEditSerialMode' . $slot}->set_active(1); }
-        elsif ($mode eq 'HostDevice') { $gui{'comboboxEditSerialMode' . $slot}->set_active(2); }
-        elsif ($mode eq 'RawFile') { $gui{'comboboxEditSerialMode' . $slot}->set_active(3); }
-        else { $gui{'comboboxEditSerialMode' . $slot}->set_active(0); }
-        $gui{'checkbuttonEditSerialPipe' . $slot}->set_active(&bl(ISerialPort_getServer($ISerialPort)));
+    if ($irq == 4 and uc($ioport) eq '3F8') { $gui{comboboxEditSerialPortNum}->set_active(0); }
+    elsif ($irq == 3 and uc($ioport) eq '2F8') { $gui{comboboxEditSerialPortNum}->set_active(1); }
+    elsif ($irq == 4 and uc($ioport) eq '3E8') { $gui{comboboxEditSerialPortNum}->set_active(2); }
+    elsif ($irq == 3 and uc($ioport) eq '2E8') { $gui{comboboxEditSerialPortNum}->set_active(3); }
+    else {
+        $gui{comboboxEditSerialPortNum}->set_active(4);
+        $gui{entryEditSerialIRQ}->set_sensitive(1);
+        $gui{entryEditSerialIO}->set_sensitive(1);
     }
 
+    # Setting the pipe must come before setting the serial mode
+    $gui{checkbuttonEditSerialPipe}->set_active(&bl(ISerialPort_getServer($ISerialPort)));
+
+    if ($mode eq 'HostPipe') { $gui{comboboxEditSerialMode}->set_active(1); }
+    elsif ($mode eq 'HostDevice') { $gui{comboboxEditSerialMode}->set_active(2); }
+    elsif ($mode eq 'RawFile') { $gui{comboboxEditSerialMode}->set_active(3); }
+    elsif ($mode eq 'TCP') { $gui{comboboxEditSerialMode}->set_active(4); }
+    else { $gui{comboboxEditSerialMode}->set_active(0); }
+
+    $gui{checkbuttonEditSerialPipe}->set_active(&bl(ISerialPort_getServer($ISerialPort)));
+
     &busy_pointer($gui{dialogEdit}, 0);
     &addrow_log('I/O Port settings complete.');
 }
 
 # Whether the serial port is enabled
 sub serial_toggle {
-    my ($widget, $checkbutton) = @_;
+    my ($widget) = @_;
 
     if ($vmc{SessionType} eq 'WriteLock') {
         my $state = $widget->get_active();
-        my ($ISerialPort, $num) = &serial_callout($checkbutton);
+        my ($ISerialPort, $port) = &serial_callout();
         ISerialPort_setEnabled($ISerialPort, $state);
-        $gui{'tableEditSerial' . $num}->set_sensitive($state);
+        $gui{tableEditSerial}->set_sensitive($state);
     }
 }
 
@@ -91,14 +95,13 @@ sub parallel_toggle {
 
 # The IRQ to use for the serial port
 sub serial_irq {
-    my ($widget, $focus, $checkbutton) = @_;
+    my ($widget, $focus) = @_;
 
     if ($vmc{SessionType} eq 'WriteLock') {
-        $checkbutton = $focus if (!$checkbutton);
         my $irq = int($widget->get_text());
         $irq = 255 if ($irq > 255);
         $irq = 15 if ($irq > 15 and !$gui{checkbuttonEditSysAPIC}->get_active());
-        my ($ISerialPort, $num) = &serial_callout($checkbutton);
+        my ($ISerialPort, $port) = &serial_callout();
         ISerialPort_setIRQ($ISerialPort, $irq);
     }
 
@@ -119,15 +122,14 @@ sub parallel_irq {
 
 # The IO Port to use for the serial port
 sub serial_ioport {
-    my ($widget, $focus, $checkbutton) = @_;
+    my ($widget) = @_;
 
     if ($vmc{SessionType} eq 'WriteLock') {
-        $checkbutton = $focus if (!$checkbutton);
         my $ioport = $widget->get_text();
         $ioport = 'FFFF' if (hex($ioport) > 65535);
 
         if ($ioport) {
-            my ($ISerialPort, $num) = &serial_callout($checkbutton);
+            my ($ISerialPort, $port) = &serial_callout();
             ISerialPort_setIOBase($ISerialPort, hex($ioport));
         }
     }
@@ -135,7 +137,7 @@ sub serial_ioport {
     return 0;
 }
 
-# The parallel port IO PORt
+# The parallel port IO PORT
 sub parallel_ioport {
     if ($vmc{SessionType} eq 'WriteLock') {
         my $ioport = $gui{entryEditParallelIO}->get_text();
@@ -148,33 +150,59 @@ sub parallel_ioport {
 
 # The serial port mode
 sub serial_portmode {
-    my ($widget, $checkbutton) = @_;
+    my ($widget) = @_;
+    my $vhost = &vhost();
 
+    # For all serial modes except disconnected, it needs a path set
+    # before it can be activated, otherwise it errors. We try to do our best with
+    # defaults here
     if ($vmc{SessionType} eq 'WriteLock') {
-        my ($ISerialPort, $num) = &serial_callout($checkbutton);
+        my ($ISerialPort, $port) = &serial_callout();
+        my $mode = &getsel_combo($widget, 0);
+
+        if ($mode eq 'RawFile') {
+            $$vhost{os} =~ m/^WINDOWS/i ? $gui{entryEditSerialPath}->set_text("\%temp\%\\raw-$vmc{Name}-ser" . ($port + 1))
+                                        : $gui{entryEditSerialPath}->set_text("/tmp/raw-$vmc{Name}-ser" . ($port + 1));
+        }
+        elsif ($mode eq 'HostPipe') {
+            $$vhost{os} =~ m/^WINDOWS/i ? $gui{entryEditSerialPath}->set_text("\\\\.\\pipe\\pipe-$vmc{Name}-ser" . ($port + 1))
+                                        : $gui{entryEditSerialPath}->set_text("/tmp/pipe-$vmc{Name}-ser" . ($port + 1));
+        }
+        elsif ($mode eq 'HostDevice') {
+            my $device = '/dev/null';
+            if ($$vhost{os} =~ m/Linux/i) { $device = '/dev/ttyS' . $port; }
+            elsif ($$vhost{os} =~ m/WINDOWS/i) { $device = "COM" . ($port + 1) . ':'; }
+            elsif ($$vhost{os} =~ m/Darwin/i) { $device = '/dev/null'; }
+            elsif ($$vhost{os} =~ m/Solaris/i) { $device = '/dev/cua'; }
+            elsif ($$vhost{os} =~ m/FreeBSD/i) { $device = '/dev/cuad' . $port; }
+            $gui{entryEditSerialPath}->set_text($device);
+        }
+        elsif ($mode eq 'TCP') { $gui{entryEditSerialPath}->set_text('65000'); }
+
+        $gui{entryEditSerialPath}->signal_emit('activate');
         ISerialPort_setHostMode($ISerialPort, &getsel_combo($widget, 0));
     }
 }
 
 # Whether to automatically create a PIPE when opening the serial port
 sub serial_createpipe {
-    my ($widget, $checkbutton) = @_;
-    
+    my ($widget) = @_;
+
     if ($vmc{SessionType} eq 'WriteLock') {
-        my ($ISerialPort, $num) = &serial_callout($checkbutton);
-        ISerialPort_setServer($ISerialPort, $widget->get_active());
+        my ($ISerialPort, $port) = &serial_callout();
+        ISerialPort_setServer($ISerialPort, !$widget->get_active());
     }
 }
 
 # The serial port path
 sub serial_portpath {
-    my ($widget, $focus, $checkbutton) = @_;
+    my ($widget) = @_;
 
     if ($vmc{SessionType} eq 'WriteLock') {
-        $checkbutton = $focus if (!$checkbutton);
+
         my $path = $widget->get_text();
         if ($path) {
-            my ($ISerialPort, $num) = &serial_callout($checkbutton);
+            my ($ISerialPort, $port) = &serial_callout();
             ISerialPort_setPath($ISerialPort, $path);
         }
     }
@@ -194,38 +222,38 @@ sub parallel_portpath {
 
 # The COM port number to use for the serial port
 sub serial_portnum {
-    my ($widget, $checkbutton) = @_;
+    my ($widget) = @_;
 
     if ($vmc{SessionType} eq 'WriteLock') {
         my $portnum = &getsel_combo($widget, 0);
-        my ($ISerialPort, $num) = &serial_callout($checkbutton);
+        my ($ISerialPort, $port) = &serial_callout();
 
         if ($portnum eq 'COM1') {
-            $gui{'entryEditSerialIRQ' . $num}->set_text('4');
-            $gui{'entryEditSerialIO' . $num}->set_text('3F8');
+            $gui{entryEditSerialIRQ}->set_text('4');
+            $gui{entryEditSerialIO}->set_text('3F8');
         }
         elsif ($portnum eq 'COM2') {
-            $gui{'entryEditSerialIRQ' . $num}->set_text('3');
-            $gui{'entryEditSerialIO' . $num}->set_text('2F8');
+            $gui{entryEditSerialIRQ}->set_text('3');
+            $gui{entryEditSerialIO}->set_text('2F8');
         }
         elsif ($portnum eq 'COM3') {
-            $gui{'entryEditSerialIRQ' . $num}->set_text('4');
-            $gui{'entryEditSerialIO' . $num}->set_text('3E8');
+            $gui{entryEditSerialIRQ}->set_text('4');
+            $gui{entryEditSerialIO}->set_text('3E8');
         }
         elsif ($portnum eq 'COM4') {
-            $gui{'entryEditSerialIRQ' . $num}->set_text('3');
-            $gui{'entryEditSerialIO' . $num}->set_text('2E8');
+            $gui{entryEditSerialIRQ}->set_text('3');
+            $gui{entryEditSerialIO}->set_text('2E8');
         }
 
         if ($portnum ne 'Custom') {
-            &serial_irq($gui{'entryEditSerialIRQ' . $num}, 0, $checkbutton);
-            &serial_ioport($gui{'entryEditSerialIO' . $num}, 0, $checkbutton);
-            $gui{'entryEditSerialIRQ' . $num}->set_sensitive(0);
-            $gui{'entryEditSerialIO' . $num}->set_sensitive(0);
+            &serial_irq($gui{entryEditSerialIRQ});
+            &serial_ioport($gui{entryEditSerialIO});
+            $gui{entryEditSerialIRQ}->set_sensitive(0);
+            $gui{entryEditSerialIO}->set_sensitive(0);
         }
         else {
-            $gui{'entryEditSerialIRQ' . $num}->set_sensitive(1);
-            $gui{'entryEditSerialIO' . $num}->set_sensitive(1);
+            $gui{entryEditSerialIRQ}->set_sensitive(1);
+            $gui{entryEditSerialIO}->set_sensitive(1);
         }
     }
 }
@@ -264,16 +292,14 @@ sub parallel_portnum {
 
 }
 
+# Gets the current working serial port number
 sub serial_callout {
-    my ($checkbutton) = @_;
+    my $port = &getsel_combo($gui{comboboxEditIOSelectedSerial}, 1);
 
     if ($vmc{SessionType} eq 'WriteLock') {
-        my $slot = 0;
-        if ($checkbutton eq $gui{checkbuttonEditSerialEnable1}) { $slot = 0; }
-        elsif ($checkbutton eq $gui{checkbuttonEditSerialEnable2}) { $slot = 1; }
-        my $ISerialPort = IMachine_getSerialPort($vmc{IMachine}, $slot);
-        return $ISerialPort, $slot+1;
+        my $ISerialPort = IMachine_getSerialPort($vmc{IMachine}, $port);
+        return $ISerialPort, $port;
     }
 }
 
-1;
\ No newline at end of file
+1;
