@@ -70,12 +70,14 @@ $gui{img}{ctr}{SAS} = $gui{img}{ctr}{SATA};
 $gui{img}{ctr}{SCSI} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/scsi_16px.png");
 $gui{img}{ctr}{Floppy} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/floppy_16px.png");
 $gui{img}{ctr}{USB} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/usb_16px.png");
+$gui{img}{ctr}{PCIe} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/nvme_16px.png");
 # Snapshot icons
 $gui{img}{SnapshotOffline} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/snapshot_offline_16px.png");
 $gui{img}{SnapshotOnline} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/snapshot_online_16px.png");
 $gui{img}{SnapshotCurrent} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/machine_16px.png");
 # OS Icons
 $gui{img}{OtherOS} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/os/Other.png");
+$gui{img}{OtherOS64} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/os/Other_64.png");
 # Other Icons
 $gui{img}{VMGroup} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/vm_group_16px.png");
 $gui{img}{DirIcon} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/sf_16px.png");
@@ -110,6 +112,7 @@ our %signal = (fam                => $gui{comboboxNewOSFam}->signal_connect(chan
                veredit            => $gui{comboboxEditGenOSVer}->signal_connect(changed => \&gen_osver, $gui{comboboxEditGenOSFam}),
                audiodrv           => $gui{comboboxEditAudioDriver}->signal_connect(changed => \&audio_driver),
                audioctr           => $gui{comboboxEditAudioCtr}->signal_connect(changed => \&audio_ctr),
+               audiocodec         => $gui{comboboxEditAudioCodec}->signal_connect(changed => \&audio_codec),
                netadaptype        => $gui{comboboxEditNetType}->signal_connect(changed => \&net_adapter_type, $gui{checkbuttonEditNetEnable}),
                netattach          => $gui{comboboxEditNetAttach}->signal_connect(changed => \&net_attach, $gui{checkbuttonEditNetEnable}),
                netname            => $gui{comboboxEditNetName}->signal_connect(changed => \&net_name, $gui{checkbuttonEditNetEnable}),
@@ -134,7 +137,7 @@ $gui{treeviewEvalConfig}->get_selection->set_mode('GTK_SELECTION_NONE');
     my @winlist = ($gui{windowMain});
 
     # Set the transient window's (ie parent window) sensitivity on
-    sub transwin_sens_on() {
+    sub transwin_sens_on {
         my ($win) = @_;
         pop @winlist;
         my $transwin = $win->get_transient_for();
@@ -142,7 +145,7 @@ $gui{treeviewEvalConfig}->get_selection->set_mode('GTK_SELECTION_NONE');
     }
 
     # Set the transient window's (ie parent window) sensitivity off
-    sub transwin_sens_off() {
+    sub transwin_sens_off {
         my ($win) = @_;
         my $transwin = $winlist[$#winlist]; # Get the last opened window
         push @winlist, $win;
@@ -152,7 +155,7 @@ $gui{treeviewEvalConfig}->get_selection->set_mode('GTK_SELECTION_NONE');
 }
 
 # Ghosts window and optionally sets pointer
-sub busy_window() {
+sub busy_window {
     my ($window, $sens, $pointer) = @_;
     $window->set_sensitive($sens);
 
@@ -163,14 +166,14 @@ sub busy_window() {
 }
 
 # Busy the pointer only
-sub busy_pointer() {
+sub busy_pointer {
     my ($window, $pointer) = @_;
     if ($pointer) { eval { $window->window->set_cursor(Gtk2::Gdk::Cursor->new('watch')); }; }
     else { eval{ $window->window->set_cursor(undef); }; }
     Gtk2->main_iteration() while Gtk2->events_pending();
 }
 
-sub handle_bioslogofilechooser() {
+sub handle_bioslogofilechooser {
     my ($basedir, $filename) = @_;
     if ($basedir and $filename) {
         $gui{entryEditSysLogoPath}->set_text(&rcatfile($basedir, $filename));
@@ -178,7 +181,7 @@ sub handle_bioslogofilechooser() {
     }
 }
 
-sub handle_videofilechooser() {
+sub handle_videofilechooser {
     my ($basedir, $filename) = @_;
     if ($basedir and $filename) {
         $gui{entryEditDispCapturePath}->set_text(&rcatfile($basedir, $filename));
@@ -186,7 +189,7 @@ sub handle_videofilechooser() {
     }
 }
 
-sub handle_machinefolderchooser() {
+sub handle_machinefolderchooser {
     my ($location, $filearrayref) = @_;
     my $file = ${$filearrayref}[0]->{FileName};
     my $type = ${$filearrayref}[0]->{Type};
@@ -197,7 +200,7 @@ sub handle_machinefolderchooser() {
     $gui{entryVBPrefsGenMachineFolder}->set_text($location) if ($location);
 }
 
-sub handle_autostartfolderchooser() {
+sub handle_autostartfolderchooser {
     my ($location, $filearrayref) = @_;
     my $file = ${$filearrayref}[0]->{FileName};
     my $type = ${$filearrayref}[0]->{Type};
@@ -208,7 +211,7 @@ sub handle_autostartfolderchooser() {
     $gui{entryVBPrefsGenAutostartDBFolder}->set_text($location) if ($location);
 }
 
-sub handle_snapshotfolderchooser() {
+sub handle_snapshotfolderchooser {
     my ($location, $filearrayref) = @_;
     my $file = ${$filearrayref}[0]->{FileName};
     my $type = ${$filearrayref}[0]->{Type};
@@ -220,7 +223,7 @@ sub handle_snapshotfolderchooser() {
     &gen_snapfolder();
 }
 
-sub handle_sharedfolderchooser() {
+sub handle_sharedfolderchooser {
     my ($location, $filearrayref) = @_;
     my $file = ${$filearrayref}[0]->{FileName};
     my $type = ${$filearrayref}[0]->{Type};
@@ -233,7 +236,7 @@ sub handle_sharedfolderchooser() {
 
 sub handle_vboxfilechooser {
     my ($basedir, $filename) = @_;
-    
+
     if ($basedir and $filename) {
         my $vboxfile = &rcatfile($basedir, $filename);
         my $IMachine = IVirtualBox_openMachine($gui{websn}, $vboxfile);
@@ -265,9 +268,9 @@ sub handle_importapplfilechooser {
 
 
 # Sets up the file chooser for selecting a BMP files
-sub show_bioslogofilechooser() {
+sub show_bioslogofilechooser {
     my ($vol, $dir, $file) = &rsplitpath($gui{entryEditSysLogoPath}->get_text());
-    
+
     &show_remotefilechooser_window({title     => "Choose BMP file on $endpoint",
                                     basedir   => $vol . $dir,
                                     filename  => $file,
@@ -277,9 +280,9 @@ sub show_bioslogofilechooser() {
 }
 
 # Sets up the file chooser for selecting the video file directory
-sub show_videofilechooser() {
+sub show_videofilechooser {
     my ($vol, $dir, $file) = &rsplitpath($gui{entryEditDispCapturePath}->get_text());
-    
+
     &show_remotefilechooser_window({title     => "Choose capture file on $endpoint",
                                     basedir   => $vol . $dir,
                                     filename  => $file,
@@ -301,7 +304,7 @@ sub show_exportapplfilechooser {
 }
 
 # Sets up the file chooser for selecting the appliance
-sub show_importapplfilechooser() {
+sub show_importapplfilechooser {
     &show_remotefilechooser_window({title     => "Choose OVF/OVA file on $endpoint",
                                     basedir   => $gui{entryVBPrefsGenMachineFolder}->get_text(),
                                     filename  => '',
@@ -311,7 +314,7 @@ sub show_importapplfilechooser() {
 }
 
 # Sets up the file chooser for selecting a VBOX file
-sub show_vboxfilechooser() {
+sub show_vboxfilechooser {
     &show_remotefilechooser_window({title     => "Choose guest to add on $endpoint",
                                     basedir   => $gui{entryVBPrefsGenMachineFolder}->get_text(),
                                     filename  => '',
@@ -321,7 +324,7 @@ sub show_vboxfilechooser() {
 }
 
 # Sets up the file chooser for selecting a medium (or any file actually)
-sub show_vmmfilechooser() {
+sub show_vmmfilechooser {
     &show_remotefilechooser_window({title     => "Choose Media Images on $endpoint",
                                     basedir   => $gui{entryVBPrefsGenMachineFolder}->get_text(),
                                     filename  => '',
@@ -331,7 +334,7 @@ sub show_vmmfilechooser() {
 }
 
 # Sets up the file chooser for selecting the default machine directory
-sub show_machinefolderchooser() {
+sub show_machinefolderchooser {
     &show_remotefilechooser_window({title     => "Choose Machine Folder on $endpoint",
                                     basedir   => $gui{entryVBPrefsGenMachineFolder}->get_text(),
                                     filename  => '',
@@ -341,7 +344,7 @@ sub show_machinefolderchooser() {
 }
 
 # Sets up the file chooser for selecting the autostart database directory
-sub show_autostartfolderchooser() {
+sub show_autostartfolderchooser {
     &show_remotefilechooser_window({title     => "Choose Autostart Database Folder on $endpoint",
                                     basedir   => $gui{entryVBPrefsGenAutostartDBFolder}->get_text(),
                                     filename  => '',
@@ -351,7 +354,7 @@ sub show_autostartfolderchooser() {
 }
 
 # Sets up the file chooser for selecting the snapshot directory
-sub show_snapshotfolderchooser() {
+sub show_snapshotfolderchooser {
     &show_remotefilechooser_window({title     => "Choose Snapshot Folder on $endpoint",
                                     basedir   => $gui{entryEditGenSnapFolder}->get_text(),
                                     filename  => '',
@@ -362,7 +365,7 @@ sub show_snapshotfolderchooser() {
 
 
 # Sets up the file chooser for selecting a shared folder
-sub show_sharedfolderchooser() {
+sub show_sharedfolderchooser {
     &show_remotefilechooser_window({title     => "Choose Shared Folder on $endpoint",
                                     basedir   => $gui{entrySharedFolderPath}->get_text(),
                                     filename  => '',
@@ -371,7 +374,7 @@ sub show_sharedfolderchooser() {
                                     handler   => \&handle_sharedfolderchooser});
 }
 
-sub show_remotefilechooser_window() {
+sub show_remotefilechooser_window {
     my ($param) = @_;
     my $vhost = &vhost();
 
@@ -405,7 +408,7 @@ sub show_remotefilechooser_window() {
         my $location = $gui{entryRemoteFileChooserLocation}->get_text();
         my $filename = $gui{entryRemoteFileChooserFile}->get_text();
         if ($$param{mode} eq 'file') { &{$$param{handler}}($location, $filename) }
-        else {        
+        else {
             my $filearrayref;
             $filearrayref = &getsel_list_remotefiles();
             &{$$param{handler}}($location, $filearrayref);
@@ -414,15 +417,15 @@ sub show_remotefilechooser_window() {
 
 }
 
-sub refresh_remotefilechooser() { &fill_list_remotefiles($gui{entryRemoteFileChooserLocation}->get_text(), $gui{entryRemoteFileChooserFilter}->get_text()); }
+sub refresh_remotefilechooser { &fill_list_remotefiles($gui{entryRemoteFileChooserLocation}->get_text(), $gui{entryRemoteFileChooserFilter}->get_text()); }
 
-sub cdup_remotefilechooser() {
+sub cdup_remotefilechooser {
     IVFSExplorer_cdUp($gui{IVFSExplorer});
     &fill_list_remotefiles(IVFSExplorer_getPath($gui{IVFSExplorer}), $gui{entryRemoteFileChooserFilter}->get_text());
 }
 
 # Display a progress window for tasks which can take a long time
-sub show_progress_window() {
+sub show_progress_window {
     my ($IProgress, $msg) = @_;
     my $resultcode = 0;
     my $timer = 0;
@@ -449,7 +452,7 @@ sub show_progress_window() {
                 return 0;
             }
             else {
-                
+
                 $gui{progressbar}->set_fraction($percent * 0.01);
                 if ($percent > 10 and $secsremaining > 0) {
                     my $humantime = &secs_to_humantime($secsremaining);
@@ -469,7 +472,7 @@ sub show_progress_window() {
 }
 
 # Restore RemoteBox's window position to the last save position
-sub restore_window_pos() {
+sub restore_window_pos {
     my ($winname) = @_;
 
     if ($prefs{"WINPOS_$winname"}) {
@@ -565,7 +568,7 @@ sub combobox_set_active_text {
 }
 
 # Adds appropriate units to a spinbox when specifying memory or disk
-sub spinbox_bytes_out() {
+sub spinbox_bytes_out {
     my ($widget) = @_;
     my $txt = $widget->get_text();
     my $adjustment = $widget->get_adjustment();
@@ -591,7 +594,7 @@ sub spinbox_bytes_out() {
 }
 
 # Parses the input and assumes m is mega, g is Giga and t is tera
-sub spinbox_bytes_in() {
+sub spinbox_bytes_in {
     my ($widget) = @_;
     my $txt = $widget->get_text();
 
@@ -616,7 +619,7 @@ sub spinbox_bytes_in() {
     return 0;
 }
 
-sub spinbox_time_in() {
+sub spinbox_time_in {
     my ($widget) = @_;
     my $txt = $widget->get_text();
 
@@ -642,7 +645,7 @@ sub spinbox_time_in() {
 }
 
 # Adds appropriate units to a spinbox when specifying time
-sub spinbox_time_out() {
+sub spinbox_time_out {
     my ($widget) = @_;
     my $txt = $widget->get_text();
     my $adjustment = $widget->get_adjustment();
@@ -668,7 +671,7 @@ sub spinbox_time_out() {
 }
 
 # Adds appropriate units to a spinbox when specifying percent
-sub spinbox_pc_out() {
+sub spinbox_pc_out {
     my ($widget) = @_;
     my $adjustment = $widget->get_adjustment();
     my $adjval = int($adjustment->get_value());
@@ -677,7 +680,7 @@ sub spinbox_pc_out() {
 }
 
 # Handles a percent when specified in a spinbox
-sub spinbox_pc_in() {
+sub spinbox_pc_in {
     my ($widget) = @_;
     my $txt = $widget->get_text();
 
@@ -685,7 +688,7 @@ sub spinbox_pc_in() {
         $txt =~ s/[^\d.]//g; # Strip all except digits and .
         $widget->get_adjustment->set_value(int($txt));
     }
-   
+
     return 0;
 }
 
@@ -696,46 +699,46 @@ sub heartbeat {
     return 1; # Return 1 to stop the timer from being removed
 }
 
-sub secs_to_humantime() { 
-    my ($time) = @_; 
-    my $hours = int($time / 3600); 
-    $time -= ($hours * 3600); 
-    my $mins = int($time / 60); 
-    my $secs = $time % 60; 
+sub secs_to_humantime {
+    my ($time) = @_;
+    my $hours = int($time / 3600);
+    $time -= ($hours * 3600);
+    my $mins = int($time / 60);
+    my $secs = $time % 60;
     $hours = '0' . $hours if ($hours < 10);
     $mins = '0' . $mins if ($mins < 10);
     $secs = '0' . $secs if ($secs < 10);
 
-    return "$hours:$mins:$secs"; 
+    return "$hours:$mins:$secs";
 }
 
 # Returns truth according to virtualbox which can take the form of
 # null, Null, False, false (being 0) and True, true being 1
-sub bl() { ($_[0] =~ m/^[t|T]/) // return 1 }
+sub bl { ($_[0] =~ m/^[t|T]/) // return 1 }
 
 # Converts a path to its canonical form
-sub rcanonpath() {
+sub rcanonpath {
     my $vhost = &vhost();
     unless ($$vhost{os} =~ m/^WINDOWS/i) { return File::Spec::Unix->canonpath(@_); }
     else { return File::Spec::Win32->canonpath(@_); }
 }
 
 # Concatenates a file onto a path
-sub rcatfile() {
+sub rcatfile {
     my $vhost = &vhost();
     unless ($$vhost{os} =~ m/^WINDOWS/i) { return File::Spec::Unix->catfile(@_); }
     else { return File::Spec::Win32->catfile(@_); }
 }
 
 # Concatenates a dir onto a path
-sub rcatdir() {
+sub rcatdir {
     my $vhost = &vhost();
     unless ($$vhost{os} =~ m/^WINDOWS/i) { return File::Spec::Unix->catdir(@_); }
     else { return File::Spec::Win32->catdir(@_); }
 }
 
 # Splits a path into volume, dir, file
-sub rsplitpath() {
+sub rsplitpath {
     my $vhost = &vhost();
     unless ($$vhost{os} =~ m/^WINDOWS/i) { return File::Spec::Unix->splitpath(@_); }
     else { return File::Spec::Win32->splitpath(@_); }
