@@ -79,7 +79,10 @@ our (%gui, %vmc, %signal, %prefs);
         $vtx .= 'PAE/NX  ' if (IMachine_getCPUProperty($$gref{IMachine}, 'PAE') eq 'true');
         $vtx .= 'Nested Paging  ' if (IMachine_getHWVirtExProperty($$gref{IMachine}, 'NestedPaging') eq 'true');
         $vtx ? &addrow_details($sysiter, 1, ' Acceleration:', 2, $vtx) : &addrow_details($sysiter, 1, ' Acceleration:', 2, '<None Enabled>');
-        &addrow_details($sysiter, 1, ' Paravirtualization:', 2, IMachine_getParavirtProvider($$gref{IMachine}));
+        my $paravirt = 'Configured: ' . IMachine_getParavirtProvider($$gref{IMachine}) . ', Effective: ' . IMachine_getEffectiveParavirtProvider($$gref{IMachine});
+        #&addrow_details($sysiter, 1, ' Paravirtualization:', 2, IMachine_getParavirtProvider($$gref{IMachine}));
+
+        &addrow_details($sysiter, 1, ' Paravirtualization:', 2, $paravirt);
         $gui{treeviewDetails}->expand_row($treemodel->get_path($sysiter), 1) if ($prefs{EXPANDDETSYS});
 
         my $dispiter = &addrow_details(undef, 0, $gui{img}{CatDisp}, 1, 'Display', 3, 800, 4, 0.0, 5, 'EXPANDDETDISP');
@@ -669,56 +672,117 @@ sub onsel_list_remotefiles_single {
 # to expand
 sub onsel_list_editstorage {
     my $storref = &getsel_list_editstorage();
+    # Sensitivities for all selections
+    $gui{frameEditStorAttr}->show();
+    $gui{buttonEditStorAddAttach}->set_sensitive(1);
+    $gui{checkbuttonEditStorHotPluggable}->hide();
+    $gui{checkbuttonEditStorSSD}->hide();
+    $gui{spinbuttonEditStorPortCount}->hide();
+    $gui{checkbuttonEditStorControllerBootable}->hide();
 
     if ($$storref{IsController}) {
-        &sens_editstor_allctr();
+        # Sensitivities for all controllers
+        $gui{buttonEditStorRemoveAttach}->set_sensitive(0);
+        $gui{buttonEditStorRemoveCtr}->set_sensitive(1);
+        $gui{labelEditStorCtrName}->show();
+        $gui{entryEditStorCtrName}->show();
+        $gui{labelEditStorCtrType}->show();
+        $gui{comboboxEditStorCtrType}->show();
+        $gui{checkbuttonEditStorCache}->show();
+        $gui{labelEditStorDevPort}->hide();
+        $gui{comboboxEditStorDevPort}->hide();
+        $gui{checkbuttonEditStorLive}->hide();
+        $gui{checkbuttonEditStorControllerBootable}->show();
+        $gui{labelEditStorPortCount}->hide();
+        $gui{menuitemAttachHD}->set_sensitive(1);
+        $gui{menuitemAttachDVD}->set_sensitive(1);
+        $gui{menuitemAttachFloppy}->set_sensitive(0);
         $gui{comboboxEditStorCtrType}->signal_handler_block($signal{stortype});
         $gui{entryEditStorCtrName}->set_text($$storref{ControllerName});
         $gui{checkbuttonEditStorCache}->set_active(&bl(IStorageController_getUseHostIOCache($$storref{IStorageController})));
+        $gui{checkbuttonEditStorControllerBootable}->set_active(&bl(IStorageController_getBootable($$storref{IStorageController})));
         my $variant = IStorageController_getControllerType($$storref{IStorageController});
 
-        if ($$storref{Bus} eq 'IDE') {
-            &sens_editstor_ctr();
-            $gui{comboboxEditStorCtrType}->set_model($gui{liststoreEditStorIDECtrType});
-        }
+        if ($$storref{Bus} eq 'IDE') { $gui{comboboxEditStorCtrType}->set_model($gui{liststoreEditStorIDECtrType}); }
+        elsif ($$storref{Bus} eq 'USB') { $gui{comboboxEditStorCtrType}->set_model($gui{liststoreEditStorUSBCtrType}); }
+        elsif ($$storref{Bus} eq 'SCSI') { $gui{comboboxEditStorCtrType}->set_model($gui{liststoreEditStorSCSICtrType}); }
         elsif ($$storref{Bus} eq 'SATA') {
-            &sens_editstor_sata_ctr();
+            $gui{labelEditStorPortCount}->show();
+            $gui{spinbuttonEditStorPortCount}->show();
+            $gui{menuitemAttachFloppy}->set_sensitive(0);
             $gui{comboboxEditStorCtrType}->set_model($gui{liststoreEditStorSATACtrType});
+            $gui{spinbuttonEditStorPortCount}->set_range(1, 30);
             $gui{adjustmentEditStorPortCount}->set_value(IStorageController_getPortCount($$storref{IStorageController}));
         }
-        elsif ($$storref{Bus} eq 'Floppy') {
-            &sens_editstor_floppy_ctr();
-            $gui{comboboxEditStorCtrType}->set_model($gui{liststoreEditStorFloppyCtrType});
-        }
         elsif ($$storref{Bus} eq 'SAS') {
-            &sens_editstor_ctr();
+            $gui{labelEditStorPortCount}->show();
+            $gui{spinbuttonEditStorPortCount}->show();
+            $gui{menuitemAttachFloppy}->set_sensitive(0);
             $gui{comboboxEditStorCtrType}->set_model($gui{liststoreEditStorSASCtrType});
+            $gui{spinbuttonEditStorPortCount}->set_range(1, 254);
+            $gui{adjustmentEditStorPortCount}->set_value(IStorageController_getPortCount($$storref{IStorageController}));
         }
-        elsif ($$storref{Bus} eq 'USB') {
-            &sens_editstor_ctr();
-            $gui{comboboxEditStorCtrType}->set_model($gui{liststoreEditStorUSBCtrType});
+        elsif ($$storref{Bus} eq 'PCIe') {
+            $gui{labelEditStorPortCount}->show();
+            $gui{spinbuttonEditStorPortCount}->show();
+            $gui{menuitemAttachDVD}->set_sensitive(0);
+            $gui{menuitemAttachFloppy}->set_sensitive(0);
+            $gui{comboboxEditStorCtrType}->set_model($gui{liststoreEditStorNVMeCtrType});
+            $gui{spinbuttonEditStorPortCount}->set_range(1, 254);
+            $gui{adjustmentEditStorPortCount}->set_value(IStorageController_getPortCount($$storref{IStorageController}));
         }
-        else { # Defaults to SCSI
-            &sens_editstor_ctr();
-            $gui{comboboxEditStorCtrType}->set_model($gui{liststoreEditStorSCSICtrType});
+        else { # Default is floppy
+            $gui{menuitemAttachHD}->set_sensitive(0);
+            $gui{menuitemAttachDVD}->set_sensitive(0);
+            $gui{menuitemAttachFloppy}->set_sensitive(1);
+            $gui{comboboxEditStorCtrType}->set_model($gui{liststoreEditStorFloppyCtrType});
         }
 
         &combobox_set_active_text($gui{comboboxEditStorCtrType}, $variant);
         $gui{comboboxEditStorCtrType}->signal_handler_unblock($signal{stortype});
     }
-    else {
-        &sens_editstor_allmedia();
+    else { # This is a medium, not a controller
+        $gui{buttonEditStorRemoveAttach}->set_sensitive(1);
+        $gui{buttonEditStorRemoveCtr}->set_sensitive(0);
+        $gui{labelEditStorCtrName}->hide();
+        $gui{entryEditStorCtrName}->hide();
+        $gui{labelEditStorCtrType}->hide();
+        $gui{comboboxEditStorCtrType}->hide();
+        $gui{checkbuttonEditStorCache}->hide();
+        $gui{checkbuttonEditStorLive}->hide();
+        $gui{labelEditStorDevPort}->show();
+        $gui{comboboxEditStorDevPort}->show();
+        $gui{labelEditStorPortCount}->hide();
+        $gui{menuitemAttachHD}->set_sensitive(0);
+        $gui{menuitemAttachDVD}->set_sensitive(0);
+        $gui{menuitemAttachFloppy}->set_sensitive(0);
 
         if ($$storref{MediumType} eq 'DVD') {
             my $attach = IMachine_getMediumAttachment($vmc{IMachine}, $$storref{ControllerName}, $$storref{Port}, $$storref{Device});
+            $gui{checkbuttonEditStorLive}->show();
             $gui{checkbuttonEditStorLive}->set_active(&bl($$attach{temporaryEject}));
-            &sens_editstor_dvd();
+            $gui{menuitemAttachDVD}->set_sensitive(1);
+            # Only SATA & USB controllers support hot pluggable
+            if ($$storref{Bus} eq 'SATA' or $$storref{Bus} eq 'USB') {
+                $gui{checkbuttonEditStorHotPluggable}->set_active(&bl($$attach{hotPluggable}));
+                $gui{checkbuttonEditStorHotPluggable}->show();
+            }
+            else { $gui{checkbuttonEditStorHotPluggable}->hide(); }
+        }
+        elsif ($$storref{MediumType} eq 'Floppy') {
+            $gui{menuitemAttachFloppy}->set_sensitive(1);
         }
-        elsif ($$storref{MediumType} eq 'Floppy') { &sens_editstor_floppy(); }
         else { # Default to HD
             my $attach = IMachine_getMediumAttachment($vmc{IMachine}, $$storref{ControllerName}, $$storref{Port}, $$storref{Device});
             $gui{checkbuttonEditStorSSD}->set_active(&bl($$attach{nonRotational}));
-            &sens_editstor_hd();
+            $gui{buttonEditStorAddAttach}->set_sensitive(0);
+            $gui{checkbuttonEditStorSSD}->show();
+            # Only SATA & USB controllers support hot pluggable
+            if ($$storref{Bus} eq 'SATA' or $$storref{Bus} eq 'USB') {
+                $gui{checkbuttonEditStorHotPluggable}->set_active(&bl($$attach{hotPluggable}));
+                $gui{checkbuttonEditStorHotPluggable}->show();
+            }
+            else { $gui{checkbuttonEditStorHotPluggable}->hide(); }
         }
 
         # We also need to setup the port comboboxEditStorDevPort
@@ -745,6 +809,14 @@ sub onsel_list_editstorage {
             $gui{comboboxEditStorDevPort}->set_model($gui{liststoreEditStorDevPortFloppy});
             $gui{comboboxEditStorDevPort}->set_active($$storref{Device});
         }
+        elsif ($$storref{Bus} eq 'PCIe') {
+            $gui{comboboxEditStorDevPort}->set_model($gui{liststoreEditStorDevPortNVMe});
+            $gui{comboboxEditStorDevPort}->set_active($$storref{Port});
+        }
+        elsif ($$storref{Bus} eq 'USB') {
+            $gui{comboboxEditStorDevPort}->set_model($gui{liststoreEditStorDevPortUSB});
+            $gui{comboboxEditStorDevPort}->set_active($$storref{Port});
+        }
     }
 }
 
