@@ -57,6 +57,7 @@ sub virtualbox_logon {
         $gui{servermem} = Glib::Timeout->add(300000, \&update_server_membar);
         return 1;
     }
+    # SHOULD THIS BE DELETED????
     #else {
     #    #&show_err_msg('connect', " ($url)");
     #    return 0;
@@ -162,7 +163,7 @@ sub show_dialog_connect {
                 my $ISystemProperties = IVirtualBox_getSystemProperties($gui{websn});
                 &addrow_log("Logged onto $endpoint");
                 &addrow_log("Running VirtualBox $ver.");
-                &show_err_msg('vboxver', "\nDetected VirtualBox Version: $ver") if ($ver !~ m/^5.0/);
+                &show_err_msg('vboxver', "\nDetected VirtualBox Version: $ver") if ($ver !~ m/^5.1/);
                 &show_err_msg('noextensions') if (ISystemProperties_getDefaultVRDEExtPack($ISystemProperties) !~ m/Oracle VM VirtualBox Extension Pack/i);
                 &fill_list_guest();
                 &sens_connect(1);
@@ -741,6 +742,7 @@ sub open_remote_display {
             $osver{$$type{id}}{recommendedAudioController} = $$type{recommendedAudioController};
             $osver{$$type{id}}{familyId} = $$type{familyId};
             if (-e "$sharedir/icons/os/$$type{id}.png") { $osver{$$type{id}}{icon} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/os/$$type{id}.png"); }
+            elsif ($$type{id} =~ m/_64$/) { $osver{$$type{id}}{icon} = $gui{img}{OtherOS64}; }
             else { $osver{$$type{id}}{icon} = $gui{img}{OtherOS}; }
         }
     }
