@@ -25,6 +25,7 @@ sub show_dialog_newguest {
                             elsif ($$osfam{$b}{description} =~ m/Other/) { return -1; }
                             else  { return lc($$osfam{$a}{description}) cmp lc($$osfam{$b}{description}) }
                           } keys %{$osfam}) {
+
         my $iter = $gui{liststoreNewOSFam}->append();
         $gui{liststoreNewOSFam}->set($iter, 0, "$$osfam{$fam}{description}", 1, $fam, 2, $$osfam{$fam}{icon});
         $gui{comboboxNewOSFam}->set_active_iter($iter) if ($fam eq 'Windows');
@@ -241,9 +242,9 @@ sub create_new_guest {
     my ($newref) = @_;
     my $osver = &osver();
     my %os = %{ $$osver{$$newref{ver}} };
-    my $dvdctrname = $os{recommendedDVDStorageBus} .  ' Controller';
-    my $hdctrname = $os{recommendedHDStorageBus} . ' Controller';
-    my $floppyctrname = 'Floppy Controller' if ($os{recommendedFloppy} eq 'true');
+    my $dvdctrname = $os{recommendedDVDStorageBus};
+    my $hdctrname = $os{recommendedHDStorageBus};
+    my $floppyctrname = 'Floppy' if ($os{recommendedFloppy} eq 'true');
     my $IMachine = IVirtualBox_createMachine($gui{websn}, '', $$newref{name}, '', $$newref{ver}, 'UUID 00000000-0000-0000-0000-000000000000', '');
 
     if ($IMachine) {
@@ -325,15 +326,11 @@ sub newgen_osfam {
     $combover->signal_handler_block($signal{ver});
     $gui{liststoreNewOSVer}->clear();
 
-    foreach my $ver (sort {
-                            if    ($$osver{$a}{description} =~ m/Other/) { return 1; }
-                            elsif ($$osver{$b}{description} =~ m/Other/) { return -1; }
-                            else  { return lc($$osver{$a}{description}) cmp lc($$osver{$b}{description}) }
-                          } (@{ $$osfam{$fam}{verids} }))
+    foreach my $ver (@{ $$osfam{$fam}{verids} })
     {
         my $iter = $gui{liststoreNewOSVer}->append();
         $gui{liststoreNewOSVer}->set($iter, 0, $$osver{$ver}{description}, 1, $ver, 2, $$osver{$ver}{icon});
-        $combover->set_active_iter($iter) if ($ver eq 'Windows7_64' | $ver eq 'Ubuntu_64' | $ver eq 'Solaris11_64');
+        $combover->set_active_iter($iter) if ($ver eq 'Windows10_64' | $ver eq 'Fedora_64' | $ver eq 'Solaris11_64' | $ver eq 'FreeBSD_64' | $ver eq 'DOS');
     }
 
     $combover->set_active(0) if ($combover->get_active() == -1);
